@@ -12,7 +12,7 @@
  *
  * Also finds out IP of specified MAC
  *
- * $Id: arping.c,v 1.6 2003/08/03 22:57:59 marvin Exp $
+ * $Id: arping.c,v 1.13 2004/08/29 18:44:15 marvin Exp $
  */
 /*
  *  Copyright (C) 2000-2002 Thomas Habets <thomas@habets.pp.se>
@@ -34,19 +34,36 @@
 //#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
+#ifndef WIN32
 #include <unistd.h>
 // NOTE: try un-commenting this
 //#include <stdint.h>
 
 #include <sys/time.h>
+#include <sys/types.h>
 #include <sys/socket.h>
 
 #include <netinet/in.h>
 #include <arpa/inet.h>
-
 #include <libnet.h>
+#endif
+
+#ifdef WIN32
+#include <win32/libnet.h>
+#endif
 #include <pcap.h>
 
+#if defined(WIN32)
+#define HAVE_ESIZE_TYPES 1
+#include "win32.h"
+#include "win32/getopt.h"
+#endif
+ 
+#if defined(linux)
+#define HAVE_ESIZE_TYPES 1
+#define FINDIF 1
+#endif
+
 #ifdef HAVE_NET_BPF_H
 #include <net/bpf.h>
 #endif
@@ -54,6 +71,10 @@
 #ifndef HAVE_ESIZE_TYPES
 /*
  * let's hope we at least have these
+ * FIXME: bleh, this is not auto-detected, so fix it with os-dependent stuff
+ * like we have above for linux
+ * But this broken thing compiled on my solaris, openbsd and linux-boxes so
+ * it kinda works.
  */
 #define u_int8_t uint8_t
 #define u_int16_t uint16_t
@@ -68,7 +89,7 @@
 #define IP_ALEN 4
 #endif
 
-const float version = 2.02;
+const float version = 2.04f;
 
 static libnet_t *libnet = 0;
 
@@ -78,6 +99,7 @@ static u_int32_t srcip,dstip;
 
 static int beep = 0;
 static int verbose = 0;
+static int alsototal = 0;
 /*static int pingmac = 0; */
 static int finddup = 0;
 static unsigned int numsent = 0;
@@ -100,8 +122,13 @@ volatile int time_to_die = 0;
 static void do_libnet_init(const char *ifname)
 {
 	char ebuf[LIBNET_ERRBUF_SIZE];
+	if (verbose > 1) {
+		printf("libnet_init(%s)\n", ifname);
+	}
 	if (libnet) {
-		return;
+		/* prolly going to switch interface from temp to real */
+		libnet_destroy(libnet);
+		libnet = 0;
 	}
 	if (getuid() && geteuid()) {
 		fprintf(stderr, "arping: must run as root\n");
@@ -109,42 +136,55 @@ static void do_libnet_init(const char *ifname)
 	}
 
 	if (!(libnet = libnet_init(LIBNET_LINK,
-				   ifname,
+				   (char*)ifname,
 				   ebuf))) {
 		fprintf(stderr, "arping: libnet_init(): %s\n", ebuf);
 		exit(1);
 	}
 }
 
-const char *arping_lookupdev_default(u_int32_t srcip, u_int32_t dstip,
+/*
+ *
+ */
+static const char *arping_lookupdev_default(u_int32_t srcip, u_int32_t dstip,
 				     char *ebuf)
 {
-	static const char *ifname;
-	if (!(ifname = pcap_lookupdev(ebuf))) {
-		return 0;
+#ifdef WIN32
+	WCHAR buf[LIBNET_ERRBUF_SIZE + PCAP_ERRBUF_SIZE];
+	WCHAR* ret = (WCHAR*)pcap_lookupdev((char*)buf);
+	if (ret != NULL) {
+		wcstombs(ebuf, ret, LIBNET_ERRBUF_SIZE + PCAP_ERRBUF_SIZE);
+		return ebuf;
 	}
-	return ifname;
+	return NULL;
+#else
+	return pcap_lookupdev(ebuf);
+#endif
 }
 
 #if defined(FINDIF) && defined(linux)
-const char *arping_lookupdev(u_int32_t srcip, u_int32_t dstip, char *ebuf)
+/*
+ *
+ */
+static const char *arping_lookupdev(u_int32_t srcip, u_int32_t dstip,
+				    char *ebuf)
 {
 	FILE *f;
-	static const char buf[1024];
+	static char buf[1024];
 	char buf1[1024];
 	char buf2[1024];
 	char *p,*p2;
 	int n;
 
-	libnet_host_lookup_r(dstip,0,buf2);
-	libnet_host_lookup_r(srcip,0,buf1);
+	do_libnet_init(NULL);
+	libnet_addr2name4_r(dstip,0,buf2);
+	libnet_addr2name4_r(srcip,0,buf1);
 
 	/*
 	 * Construct and run command
 	 */
 	snprintf(buf, 1023, "/sbin/ip route get %s from %s 2>&1",
 		 buf2,buf1);
-	DEBUG(printf("%s\n",buf));
 	if (!(f = popen(buf, "r"))) {
 		goto failed;
 	}
@@ -178,12 +218,39 @@ const char *arping_lookupdev(u_int32_t srcip, u_int32_t dstip, char *ebuf)
 	return arping_lookupdev_default(srcip,dstip,ebuf);
 }
 #else
-const char *arping_lookupdev(u_int32_t srcip, u_int32_t dstip, char *ebuf)
+/*
+ *
+ */
+static const char *arping_lookupdev(u_int32_t srcip, u_int32_t dstip,
+				    char *ebuf)
 {
 	return arping_lookupdev_default(srcip,dstip,ebuf);
 }
 #endif
 
+
+#ifdef WIN32
+static BOOL WINAPI arping_console_ctrl_handler(DWORD dwCtrlType )
+{
+	if(verbose) {
+		printf("arping_console_ctrl_handler( %d )\n", dwCtrlType );
+	}
+	time_to_die = 1;
+
+#if 0
+	/* if SetConsoleCtrlHandler() does what I think, this isn't needed */
+	if (display == NORMAL) {
+		printf("\n--- %s statistics ---\n"
+		       "%d packets transmitted, %d packets received, %3.0f%% "
+		       "unanswered\n",target,numsent,numrecvd,
+		       100.0 - 100.0 * (float)(numrecvd)/(float)numsent);
+		       }
+#endif
+	return TRUE;
+}
+#endif
+
+
 /*
  *
  */
@@ -199,7 +266,7 @@ static void usage(int ret)
 {
 	printf("ARPing %1.2f, by Thomas Habets <thomas@habets.pp.se>\n",
 	       version);
-	printf("usage: arping [ -0aAbdFpqrRv ] [ -w <us> ] [ -S <host/ip> ] "
+	printf("usage: arping [ -0aAbdFpqrRuv ] [ -w <us> ] [ -S <host/ip> ] "
 	       "[ -T <host/ip ]\n"
 	       "              [ -s <MAC> ] [ -t <MAC> ] [ -c <count> ] "
 	       "[ -i <interface> ]\n"
@@ -208,15 +275,46 @@ static void usage(int ret)
 }
 
 /*
- *
+ * It was unclear from msdn.microsoft.com if their scanf() supported
+ * [0-9a-fA-F], so I'll stay away from it.
  */
 static int is_mac_addr(const char *p)
 {
-	unsigned int n[6];
-	if(6==sscanf(p, "%2x%x.%2x%x.%2x%x",
-		     &n[0],&n[1],&n[2],&n[3],&n[4],&n[5])){
+	/* cisco-style */
+	if (3*5-1 == strlen(p)) {
+		int c;
+		for (c = 0; c < strlen(p); c++) {
+			if ((c % 5) == 4) {
+				if ('.' != p[c]) {
+					goto checkcolon;
+				}
+			} else {
+				if (!isxdigit(p[c])) {
+					goto checkcolon;
+				}
+			}
+		}
 		return 1;
 	}
+	/* windows-style */
+	if (6*3-1 == strlen(p)) {
+		int c;
+		for (c = 0; c < strlen(p); c++) {
+			if ((c % 3) == 2) {
+				if ('-' != p[c]) {
+					goto checkcolon;
+				}
+			} else {
+				if (!isxdigit(p[c])) {
+					goto checkcolon;
+				}
+			}
+		}
+		return 1;
+	}
+
+ checkcolon:
+	/* unix */
 	return strchr(p, ':') ? 1 : 0;
 }
 
@@ -235,6 +333,8 @@ static int get_mac_addr(const char *in,
 		return 1;
 	} else if(6 == sscanf(in, "%2x%x.%2x%x.%2x%x",n0,n1,n2,n3,n4,n5)) {
 		return 1;
+	} else if(6 == sscanf(in, "%x-%x-%x-%x-%x-%x",n0,n1,n2,n3,n4,n5)) {
+		return 1;
 	}
 	return 0;
 }
@@ -448,9 +548,13 @@ static void pingip_recv(const char *unused, struct pcap_pkthdr *h,
 					       (c<5)?':':' ');
 				}
 				
-				printf("(%s): index=%d time=%s",
+				printf("(%s): index=%d",
 				       libnet_addr2name4(ip,0),
-				       numrecvd,
+				       numrecvd);
+				if (alsototal) {
+					printf("/%u", numsent-1);
+				}
+				printf(" time=%s",
 				       tv2str(&lastpacketsent,
 					      &arrival,buf));
 				break; }
@@ -531,7 +635,7 @@ static void pingmac_recv(const char *unused, struct pcap_pkthdr *h,
 				       (c<5)?':':')');
 			}
 			printf(": icmp_seq=%d time=%s",
-			       hicmp->icmp_seq,tv2str(&lastpacketsent,
+			       htons(hicmp->icmp_seq),tv2str(&lastpacketsent,
 						      &arrival,buf));
 			break; }
 		case RAW:
@@ -567,17 +671,55 @@ static void pingmac_recv(const char *unused, struct pcap_pkthdr *h,
  */
 static void ping_recv(pcap_t *pcap,u_int32_t packetwait, pcap_handler func)
 {
-	struct timeval tv,tv2;
-	char done = 0;
-	fd_set fds;
+       struct timeval tv,tv2;
+       char done = 0;
+#ifndef WIN32
+       fd_set fds;
+#endif
 
+       if(verbose>3) {
+               printf("arping: receiving packets...\n");
+       }
+
+#ifdef WIN32
+       /* windows won't let us do select() */
+       if (-1 == gettimeofday(&tv2,NULL)) {
+	       fprintf(stderr, "arping: gettimeofday(): %s\n",
+		       strerror(errno));
+               sigint(0);
+       }
+       while (!done && !time_to_die) {
+	       struct pcap_pkthdr *pkt_header;
+	       u_char *pkt_data;
+	       if (pcap_next_ex(pcap, &pkt_header, &pkt_data) == 1) {
+		       func(pcap, pkt_header, pkt_data);
+	       }
+	       if (-1 == gettimeofday(&tv,NULL)) {
+		       fprintf(stderr, "arping: "
+			       "gettimeofday(): %s\n",
+			       strerror(errno));
+		       sigint(0);
+	       }
+               /*
+                * setup next timeval, not very exact
+                */
+               tv.tv_sec  = (packetwait / 1000000)
+		       - (tv.tv_sec - tv2.tv_sec);
+	       tv.tv_usec = (packetwait % 1000000)
+		       - (tv.tv_usec - tv2.tv_usec);
+	       while (tv.tv_usec < 0) {
+		       tv.tv_sec--;
+		       tv.tv_usec += 1000000;
+	       }
+	       usleep(10);
+	       if (tv.tv_sec < 0) {
+		       done=1;
+	       }
+       }
+#else
 	tv.tv_sec = packetwait / 1000000;
 	tv.tv_usec = packetwait % 1000000;
 
-	if(verbose>3) {
-		printf("arping: receiving packets...\n");
-	}
-
 	for (;!done;) {
 		int sr;
 		FD_ZERO(&fds);
@@ -656,6 +798,7 @@ static void ping_recv(pcap_t *pcap,u_int32_t packetwait, pcap_handler func)
 		fprintf(stderr, "arping: select(delay): %s\n",strerror(errno));
 		sigint(0);
 	}
+#endif
 }
 
 /*
@@ -670,7 +813,7 @@ int main(int argc, char **argv)
 	int srcip_given = 0;
 	int srcmac_given = 0;
 	int dstip_given = 0;
-	char *ifname = NULL;
+	const char *ifname = NULL;
 	char *parm;
 	int c;
 	unsigned int maxcount = -1;
@@ -687,7 +830,7 @@ int main(int argc, char **argv)
 	memset(dstmac, 0xff, ETH_ALEN);
 	memset(ethxmas, 0xff, ETH_ALEN);
 
-	while (EOF!=(c = getopt(argc, argv, "0aAbBc:dFhi:pqrRs:S:t:T:vw:"))) {
+	while (EOF!=(c=getopt(argc, argv, "0aAbBc:dFhi:I:pqrRs:S:t:T:uvw:"))) {
 		switch(c) {
 		case '0':
 			srcip = 0;
@@ -722,10 +865,13 @@ int main(int argc, char **argv)
 			if (strchr(optarg, ':')) {
 				fprintf(stderr, "arping: If you're trying to "
 					"feed me an interface alias then you "
-					"don't really\nknow what this program "
-					"does, do you?\n");
+					"don't really\nknow what this programs"
+					" does, do you?\nUse -I if you really"
+					" mean it (undocumented on "
+					"purpose)\n");
 				exit(1);
 			}
+		case 'I': /* FALL THROUGH */
 			ifname = optarg;
 			break;
 		case 'p':
@@ -752,6 +898,7 @@ int main(int argc, char **argv)
 			for (c = 0; c < 6; c++) {
 				srcmac[c] = n[c] & 0xff;
 			}
+			srcmac_given = 1;
 			break;
 		}
 		case 'S': // set source IP, may be null for don't-know
@@ -783,8 +930,8 @@ int main(int argc, char **argv)
 			for (c = 0; c < 6; c++) {
 				dstmac[c] = n[c] & 0xff;
 			}
-			break;
 			mode = PINGIP;
+			break;
 		}
 		case 'T': // set destination IP
 			if (mode == PINGIP) {
@@ -803,6 +950,9 @@ int main(int argc, char **argv)
 			}
 			mode = PINGMAC;
 			break;
+		case 'u':
+			alsototal = 1;
+			break;
 		case 'v':
 			verbose++;
 			break;
@@ -814,7 +964,7 @@ int main(int argc, char **argv)
 		}
 	}
 
-	parm = argv[optind];
+	parm = (optind < argc) ? argv[optind] : NULL;
 
 	/*
 	 * Handle dstip_given instead of ip address after parms (-B really)
@@ -834,6 +984,10 @@ int main(int argc, char **argv)
 		}
 	}
 
+	if (!parm) {
+		usage(1);
+	}
+
 	/*
 	 *
 	 */
@@ -897,8 +1051,7 @@ int main(int argc, char **argv)
 	 */
 
 	/*
-	 * Get some good iface. FIXME: traverse routing or something to find
-	 * right one.
+	 * Get some good iface.
 	 */
 	if (!ifname) {
 		if (dont_use_arping_lookupdev) {
@@ -921,10 +1074,16 @@ int main(int argc, char **argv)
 		}
 	}
 
+	/*
+	 * Init libnet again, because we now know the interface name.
+	 * We should know it by know at least
+	 */
+	do_libnet_init(ifname);
+
 	/*
 	 * pcap init
 	 */
-	if (!(pcap = pcap_open_live(ifname, 100, promisc, 10, ebuf))) {
+	if (!(pcap = pcap_open_live((char*)ifname, 100, promisc, 10, ebuf))) {
 		fprintf(stderr, "arping: pcap_open_live(): %s\n",ebuf);
 		exit(1);
 	}
@@ -976,7 +1135,12 @@ int main(int argc, char **argv)
 			exit(1);
 		}
 	}
+#ifdef WIN32
+	SetConsoleCtrlHandler(NULL, TRUE);
+	SetConsoleCtrlHandler(arping_console_ctrl_handler, TRUE);
+#else
 	signal(SIGINT, sigint);
+#endif
 
 	if (verbose) {
 		printf("This box:   Interface: %s  IP: %s   MAC address: ",
@@ -1019,6 +1183,5 @@ int main(int argc, char **argv)
 	}
 	exit(!numrecvd);
 
-
 	return 0;
 }
